<?php

/*
    UTStatsDB
    Copyright (C) 2002-2024  Patrick Contreras / Paul Gallier

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

function sql_connect(int $dbwrite = 0) {
  global $SQLhost, $SQLport, $SQLdb, $SQLus, $SQLpw, $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      if (!isset($SQLport) || $SQLport == "")
        $SQLport = 3306;
      try {
        $link = new PDO("mysql:host=$SQLhost;port=$SQLport;dbname=$SQLdb",$SQLus,$SQLpw);
      }
      catch (PDOException $e) {
        echo "Database access error.\n";
        exit;
      }
      if (!$link) {
        echo "Database access error.\n";
        exit;
      }
      $link->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, true);
      break;
    case "mysqli":
      if (!isset($SQLport) || $SQLport == "")
        $SQLport = 3306;
      if (is_numeric($SQLport))
        $SQLsock = NULL;
      else {
        $SQLhost = NULL;
        $SQLsock = $SQLport;
        $SQLport = NULL;
      }
      $link = @mysqli_connect("$SQLhost","$SQLus","$SQLpw","$SQLdb",$SQLport,"$SQLsock");
      if (!$link || mysqli_connect_errno()) {
        echo "Database access error.\n";
        exit;
      }
      break;
    case "sqlite":
      if ($dbwrite < 2) {
        try {
          if (file_exists($SQLdb)) {
            if ($dbwrite == 1)
              $link = new SQLite3($SQLdb, SQLITE3_OPEN_READWRITE);
            else
              $link = new SQLite3($SQLdb, SQLITE3_OPEN_READONLY);
          }
          else
            $link = false;
        }
        catch (PDOException $e) {
          echo "Database access error.\n";
          exit;
        }
      }
      else
        try {
          $link = new SQLite3($SQLdb, SQLITE3_OPEN_READWRITE | SQLITE3_OPEN_CREATE );
        }
        catch (PDOException $e) {
          echo "Database access error.\n";
          exit;
        }

      if ($link !== false)
        @$link->createFunction('FROM_UNIXTIME', 'from_unixtime', 1);
      break;
    case "mssql":
      $sqlconnect = array("Database"=>"$SQLdb", "UID"=>"$SQLus", "PWD"=>"$SQLpw", "ReturnDatesAsStrings"=>true);
      $link = @sqlsrv_connect($SQLhost, $sqlconnect);
      if (!$link) {
        echo "Database access error.\n";
        exit;
      }
      break;
    default:
      echo "Database type error.\n";
      exit;
  }

  return $link;
}

function sql_query($link, $query) {
  global $uselimit, $dbtype;

  if (!isset($uselimit) || !$uselimit) { // Remove LIMIT 1 from UPDATE queries for unsupported versions
    if (!strcmp(substr($query, 0, 6), "UPDATE") && !strcmp(substr($query, -7), "LIMIT 1"))
      $query = substr($query, 0, -7);
  }
  switch (strtolower($dbtype)) {
    case "pdomysql":
      $result = $link->query($query);
      if (!$result) {
      	$err = "*Error in database query: '$query'";
      	error_log($err);
      	$errar = $link->errorInfo();
        $err = $errar[0] . " : " . $errar[1] . " : " . $errar[2];
        error_log($err);
      }
      break;
    case "mysqli":
      $result = @mysqli_query($link, "$query", MYSQLI_USE_RESULT); // Unbuffered query
      if (!$result) {
      	$err = "*Error in database query: '$query'";
      	error_log($err);
        $err = mysqli_errno($link) . ": " . mysqli_error($link);
        error_log($err);
      }
      break;
    case "sqlite":
      $query = preg_replace("/ USE INDEX \(.*\)/", "", $query);
      $query = str_replace("\'", "&#39;", $query);
      $result = @$link->query($query);
      if (!$result) {
      	$err = "*Error in database query: '$query'";
      	error_log($err);
        error_log($link->lastErrorMsg());
      }
      break;
    case "mssql":
      $query = sqlsrv_queryfix($query);
      $query = str_replace("\'", "' + char(39) + '", $query);
      $result = @sqlsrv_query($link, $query);
      if ($result == FALSE) {
      	$err = "*Error in database query: '$query'";
      	error_log($err);
      }
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
  return $result;
}

function sql_queryb($link, $query, $extra=0) {
  global $uselimit, $dbtype;

  if (!isset($uselimit) || !$uselimit) { // Remove LIMIT 1 from UPDATE queries
    if (!strcmp(substr($query, 0, 6), "UPDATE") && !strcmp(substr($query, -7), "LIMIT 1"))
      $query = substr($query, 0, -7);
  }
  switch (strtolower($dbtype)) {
    case "pdomysql":
      $result = $link->query($query);
      if (!$result) {
      	$err = "*Error in database query: '$query'";
      	error_log($err);
      	$errar = $link->errorInfo();
        $err = $errar[0] . " : " . $errar[1] . " : " . $errar[2];
        error_log($err);
      }
      break;
    case "mysqli":
      $result = @mysqli_query($link, "$query");
      if (!$result) {
      	$err = "*Error in database query: '$query'";
      	error_log($err);
        $err = mysqli_errno($link) . ": " . mysqli_error($link);
        error_log($err);
      }
      break;
    case "sqlite":
      $query = preg_replace("/ USE INDEX \(.*\)/", "", $query);
      $result = @$link->query($query);
      if (!$result) {
      	$err = "*Error in database query: '$query'";
      	error_log($err);
        error_log($link->lastErrorMsg());
      }
      break;
    case "mssql":
      $query = sqlsrv_queryfix($query);
      if ($extra == 1) {
        $params = array();
        $options = array("Scrollable"=>SQLSRV_CURSOR_STATIC);
        $result = @sqlsrv_query($link, $query, $params, $options);
      }
      else
        $result = @sqlsrv_query($link, $query);
      if (!$result) {
      	$err = "*Error in database query: '$query'";
      	error_log($err);
      }
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
  return $result;
}

function sql_fetch_row($result) {
  global $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      $row = $result->fetch(PDO::FETCH_NUM);
      break;
    case "mysqli":
      $row = @mysqli_fetch_row($result);
      break;
    case "sqlite":
      $row = @$result->fetchArray(SQLITE3_NUM);
      break;
    case "mssql":
      if ($result != FALSE) {
        $row = @sqlsrv_fetch_array($result);
        $i = 0;
        while (isset($row[$i])) {
          if (is_string($row[$i]) && $row[$i] == " ")
            $row[$i] = "";
          $i++;
        }
      }
      else
        $row = NULL;
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
  return $row;
}

function sql_fetch_assoc($result) {
  global $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      $row = $result->fetch(PDO::FETCH_ASSOC);
      break;
    case "mysqli":
      $row = @mysqli_fetch_assoc($result);
      break;
    case "sqlite":
      $row = @$result->fetchArray(SQLITE3_ASSOC);
      break;
    case "mssql":
      if ($result != FALSE)
        $row = @sqlsrv_fetch_array($result, SQLSRV_FETCH_ASSOC);
      else
        $row = NULL;
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
  return $row;
}

function sql_fetch_array($result) {
  global $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      $row = $result->fetch(PDO::FETCH_BOTH);
      break;
    case "mysqli":
      $row = @mysqli_fetch_array($result);
      break;
    case "sqlite":
      $row = @$result->fetchArray(SQLITE3_BOTH);
      break;
    case "mssql":
      if ($result != FALSE)
        $row = @sqlsrv_fetch_array($result);
      else
        $row = NULL;
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
  return $row;
}

function sql_free_result($result) {
  global $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      $result = null;
      break;
    case "mysqli":
      @mysqli_free_result($result);
      break;
    case "sqlite":
      @$result->finalize();
      break;
    case "mssql":
      if ($result != FALSE)
        @sqlsrv_free_stmt($result);
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
}

function sql_insert_id($link, $tbl) {
  global $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      $num = $link->lastInsertId();
      break;
    case "mysqli":
      $num = @mysqli_insert_id($link);
      break;
    case "sqlite":
      $num = @$link->lastInsertRowID();
      break;
    case "mssql":
      $result = @sqlsrv_query($link, "SELECT IDENT_CURRENT('$tbl') as ID");
      if ($result) {
      	$row = sql_fetch_row($result);
      	$num = $row[0];
        if (!is_numeric($num))
          $num = 0;
      }
      else
        $num = 0;
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
  return $num;
}

function sql_num_rows($result) {
  global $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql": // Not available in unbuffered mode
      $num = $result->rowCount();
      break;
    case "mysqli": // Not available in unbuffered mode
      $num = mysqli_num_rows($result);
      break;
    case "sqlite": // Not available in unbuffered mode
      $num = 0;
      while ($result->fetchArray(SQLITE3_NUM))
        $num++;
      $result->reset();
      break;
    case "mssql":
      $num = sqlsrv_num_rows($result);
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
  return $num;
}

function sql_affected_rows($link, $result) {
  global $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      $num = $result->rowCount();
      break;
    case "mysqli":
      $num = mysqli_affected_rows($link);
      break;
    case "sqlite":
      $num = $link->changes();
      break;
    case "mssql":
      $num = sqlsrv_rows_affected($result);
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
  return $num;
}

function sql_close($link) {
  global $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      $link = null;
      break;
    case "mysqli":
      mysqli_close($link);
      break;
    case "sqlite":
      $link->close();
      break;
    case "mssql":
      sqlsrv_close($link);
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
}

function sql_addslashes($link, $str) {
  global $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      $str = addslashes($str);
      break;
    case "mysqli":
      $str = mysqli_real_escape_string($link, $str);
      break;
    case "sqlite":
      $str = $link->escapeString($str);
      break;
    case "mssql":
      $str = addslashes($str);
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
  return $str;
}

function sql_error($link) {
  global $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      $errar = $link->errorInfo();
      $err = $errar[0] . " : " . $errar[1] . " : " . $errar[2];
      break;
    case "mysqli":
      $err = mysqli_error($link);
      break;
    case "mssql":
      $err = "";
      break;
    case "sqlite":
      $err = $link->lastErrorMsg();
      break;
    case "pgsql":
      $err = pg_last_error($link);
      break;
    default:
      echo "Database type error.\n";
      exit;
      break;
  }
  return $err;
}

function from_unixtime($unixtime)
{
  return "'".date('Y-m-d H:i:s', $unixtime)."'";
}

function sql_show_tables($query) {
  global $SQLhost, $SQLport, $SQLdb, $SQLus, $SQLpw, $dbtype;

  switch (strtolower($dbtype)) {
    case "pdomysql":
      if (!isset($SQLport) || $SQLport == "")
        $SQLport = 3306;
      try {
        $link = new PDO("mysql:host=$SQLhost;port=$SQLport;dbname=$SQLdb",$SQLus,$SQLpw);
      }
      catch (PDOException $e) {
        echo "Database access error.\n";
        exit;
      }
      if (!$link) {
        echo "Database access error.\n";
        exit;
      }
      $result = $link->query($query);
      $link->close();
      break;
    case "mysqli":
      if (!isset($SQLport) || $SQLport == "")
        $SQLport = 3306;
      if (is_numeric($SQLport))
        $SQLsock = NULL;
      else {
        $SQLhost = NULL;
        $SQLsock = $SQLport;
        $SQLport = NULL;
      }
      $link = @mysqli_connect("$SQLhost","$SQLus","$SQLpw","$SQLdb",$SQLport,"$SQLsock");
      if (!$link) {
        echo "Database access error.\n";
        exit;
      }
      $result = @mysqli_query($link, "$query");
      mysqli_close($link);
      break;
    case "sqlite":
      try {
        $db = new SQLite3("$SQLdb");
      }
      catch (PDOException $e) {
        echo "Database access error.\n";
        exit;
      }
      $result = @$db->query($query);
      $db->close();
      break;
    case "mssql":
      $sqlconnect = array("Database"=>"$SQLdb", "UID"=>"$SQLus", "PWD"=>"$SQLpw", "ReturnDatesAsStrings"=>true);
      $link = @sqlsrv_connect($SQLhost, $sqlconnect);
      if (!$link) {
        echo "Database access error.\n";
        exit;
      }
      $result = @sqlsrv_query($link, $query);
      sqlsrv_close($link);
      break;
    default:
      echo "Database type error.\n";
      exit;
  }
  return $result;
}

// Modified from code by Jon Jensen
function sqlite_alter_table($link, $table, $alterdefs)
{
  $result = @$link->query("SELECT sql,name,type FROM sqlite_master WHERE tbl_name = '".$table."' ORDER BY type DESC");

  if (sqlite_num_rows($result) > 0) {
    $row = @$result->fetchArray(SQLITE3_NUM);
    $tmpname = 't'.time();
    $origsql = trim(preg_replace("/[\s]+/"," ",str_replace(",",", ",preg_replace("/[\(]/","( ",$row['sql'],1))));
    $createtemptableSQL = 'CREATE TEMPORARY '.substr(trim(preg_replace("'".$table."'",$tmpname,$origsql,1)),6);
    $createindexsql = array();
    $i = 0;
    $defs = preg_split("/[,]+/",$alterdefs,-1,PREG_SPLIT_NO_EMPTY);
    $prevword = $table;

    // $oldcols = preg_split("/[,]+/",substr(trim($createtemptableSQL),strpos(trim($createtemptableSQL),'(')+1),-1,PREG_SPLIT_NO_EMPTY);
    $oldcols = array();
    $tmpcols = trim($origsql);
    $p = strpos($tmpcols, "(");
    $tmpcols = substr($tmpcols, $p + 1);
    $tmpcols = trim($tmpcols);
    $p = strpos($tmpcols, ",");

    while ($p != FALSE) {
      $n = 0;
      if (substr($tmpcols, $p - 2, 1) != "(" && substr($tmpcols, $p - 3, 1) != "(") {
        $oldcols[] = substr($tmpcols, 0, $p);
        $tmpcols = substr($tmpcols, $p + 1);
      }
      else
        $n = $p + 1;
      $p = strpos($tmpcols, ",", $n);
    }

    $newcols = array();

    for ($i=0;$i<sizeof($oldcols);$i++) {
      $colparts = preg_split("/[\s]+/",$oldcols[$i],-1,PREG_SPLIT_NO_EMPTY);
      $oldcols[$i] = $colparts[0];
      $newcols[$colparts[0]] = $colparts[0];
    }

    $newcolumns = '';
    $oldcolumns = '';
    reset($newcols);

    foreach ($newcols as $key => $val) {
      $newcolumns .= ($newcolumns?', ':'').$val;
      $oldcolumns .= ($oldcolumns?', ':'').$key;
    }

    $copytotempsql = 'INSERT INTO '.$tmpname.'('.$newcolumns.') SELECT '.$oldcolumns.' FROM '.$table;
    $dropoldsql = 'DROP TABLE '.$table;
    $createtesttableSQL = $createtemptableSQL;

    foreach ($defs as $def) {
      $defparts = preg_split("/[\s]+/", $def, -1, PREG_SPLIT_NO_EMPTY);
      $action = strtolower($defparts[0]);
      switch($action) {
      case 'add':
        if (sizeof($defparts) <= 2) {
          trigger_error('near "'.$defparts[0].($defparts[1]?' '.$defparts[1]:'').'": syntax error',E_USER_WARNING);
          return false;
        }
        $createtesttableSQL = substr($createtesttableSQL,0,strlen($createtesttableSQL)-1).',';
        for ($i = 1; $i < sizeof($defparts); $i++)
          $createtesttableSQL.=' '.$defparts[$i];
        $createtesttableSQL.=')';
        break;
      case 'change':
        if (sizeof($defparts) <= 3) {
          trigger_error('near "'.$defparts[0].($defparts[1]?' '.$defparts[1]:'').($defparts[2]?' '.$defparts[2]:'').'": syntax error',E_USER_WARNING);
          return false;
        }
        if ($severpos = strpos($createtesttableSQL,' '.$defparts[1].' ')) {
          if ($newcols[$defparts[1]] != $defparts[1]) {
            trigger_error('unknown column "'.$defparts[1].'" in "'.$table.'"',E_USER_WARNING);
            return false;
          }
          $newcols[$defparts[1]] = $defparts[2];
          $nextcommapos = strpos($createtesttableSQL,',',$severpos);
          $insertval = '';
          for ($i=2;$i<sizeof($defparts);$i++)
            $insertval.=' '.$defparts[$i];
          if ($nextcommapos)
            $createtesttableSQL = substr($createtesttableSQL,0,$severpos).$insertval.substr($createtesttableSQL,$nextcommapos);
          else
            $createtesttableSQL = substr($createtesttableSQL,0,$severpos-(strpos($createtesttableSQL,',')?0:1)).$insertval.')';
        }
        else {
          trigger_error('unknown column "'.$defparts[1].'" in "'.$table.'"',E_USER_WARNING);
          return false;
        }
        break;
      case 'drop':
        if (sizeof($defparts) < 2) {
          trigger_error('near "'.$defparts[0].($defparts[1]?' '.$defparts[1]:'').'": syntax error',E_USER_WARNING);
          return false;
        }
        if ($severpos = strpos($createtesttableSQL,' '.$defparts[1].' ')) {
          $nextcommapos = strpos($createtesttableSQL,',',$severpos);
          if ($nextcommapos)
            $createtesttableSQL = substr($createtesttableSQL,0,$severpos).substr($createtesttableSQL,$nextcommapos + 1);
          else
            $createtesttableSQL = substr($createtesttableSQL,0,$severpos-(strpos($createtesttableSQL,',')?0:1) - 1).')';
          unset($newcols[$defparts[1]]);
        }
        else {
          trigger_error('unknown column "'.$defparts[1].'" in "'.$table.'"',E_USER_WARNING);
          return false;
        }
        break;
      default:
        trigger_error('near "'.$prevword.'": syntax error',E_USER_WARNING);
        return false;
      }
      $prevword = $defparts[sizeof($defparts)-1];
    }

    // Generates a test table simply to verify that the columns specifed are valid in an sql statement
    $result = @$link->query($createtesttableSQL);
    if (!$result) {
      print("SQLite Error creating test table.<br>\n");
      return false;
    }
    $droptempsql = 'DROP TABLE '.$tmpname;
    $result = @$link->query($droptempsql);
    if (!$result) {
      print("SQLite Error dropping test table.<br>\n");
      return false;
    }

    $createnewtableSQL = 'CREATE '.substr(trim(preg_replace("'".$tmpname."'",$table,$createtesttableSQL,1)),17);
    $newcolumns = '';
    $oldcolumns = '';
    reset($newcols);

    foreach ($newcols as $key => $val) {
      $newcolumns .= ($newcolumns?', ':'').$val;
      $oldcolumns .= ($oldcolumns?', ':'').$key;
    }
    $copytonewsql = 'INSERT INTO '.$table.'('.$newcolumns.') SELECT '.$oldcolumns.' FROM '.$tmpname;

    $result = @$link->query($createtemptableSQL); // Create temp table
    if (!$result) {
      print("SQLite Error creating temp table.<br>\n");
      return false;
    }
    $result = @$link->query($copytotempsql); // Copy to table
    if (!$result) {
      print("SQLite Error copying to temp table.<br>\n");
      return false;
    }
    $result = @$link->query($dropoldsql); // Drop old table
    if (!$result) {
      print("SQLite Error dropping old table.<br>\n");
      return false;
    }

    $result = @$link->query($createnewtableSQL); // Recreate original table
    if (!$result) {
      print("SQLite Error creating original table.<br>\n");
      return false;
    }
    $result = @$link->query($copytonewsql); // Copy back to original table
    if (!$result) {
      print("SQLite Error copying to original table.<br>\n");
      return false;
    }
    $result = @$link->query($droptempsql); // Drop temp table
    if (!$result) {
      print("SQLite Error dropping temp table.<br>\n");
      return false;
    }
  }
  else {
    trigger_error('no such table: '.$table,E_USER_WARNING);
    return false;
  }

  return true;
}

function sqlsrv_queryfix($query)
{
  // Convert from LIMIT to TOP in SELECT queries
  if (!strcmp(substr($query, 0, 6), "SELECT") && strstr($query, " LIMIT ") !== false) {
  	if (($pl = strpos($query, " LIMIT ")) !== false) {
  	  $lim = substr($query, $pl + 7);
  	  if (($pc = strpos(substr($query, $pl + 7), ",")) === false)
        $query = "SELECT TOP $lim " . substr($query, 6, $pl - 6);
      else {
        $lim1 = intval(substr($lim, 0, $pc));
        $lim2 = intval(substr($lim, $pc + 1));
        $lim1 += $lim2;
        $query = "SELECT TOP $lim2 * FROM (SELECT TOP $lim1 " . substr($query, 6, $pl - 6) . ") AS mslimit";
      }
    }
  }

  // Convert FROM_UNIXTIME
/*
  if (($ut = strpos($query, "FROM_UNIXTIME(")) !== false) {
  	if (($eq = strpos(substr($query, $ut + 14), ")")) !== false) {
  	  $dt = date('Y-m-d H:i:s', substr($query, $ut + 14, $eq));
  	  $query = substr($query, 0, $ut)."'".$dt."'".substr($query, $ut + $eq + 15);
    }
  }
*/

  // Fix date queries
/*
  if (!strcmp(substr($query, 0, 6), "SELECT") && strstr($query, "DATEPART") === FALSE) {
    $daterows = array("cn_ctime", "cn_dtime", "mp_lastmatch", "gm_init", "gm_start", "sv_lastmatch", "tl_chfragssg_date", "tl_chkillssg_date", "tl_chdeathssg_date", "tl_chsuicidessg_date", "tl_chcarjacksg_date", "tl_chroadkillssg_date", "tl_chcpcapturesg_date", "tl_chflagcapturesg_date", "tl_chflagreturnsg_date", "tl_chflagkillsg_date", "tl_chbombcarriedsg_date", "tl_chbombtossedsg_date", "tl_chbombkillsg_date", "tl_chnodeconstructedsg_date", "tl_chnodeconstdestroyedsg_date", "tl_chnodedestroyedsg_date", "wp_chkillssg_dt", "wp_chdeathssg_dt", "wp_chdeathshldsg_dt", "wp_chsuicidessg_dt");
    foreach ($daterows as $daterow) {
      if (($p = strpos($query, $daterow)) !== false && substr($query, $p + strlen($daterow), 1) != "=") {
        if (strstr($query, "MAX(") === false)
          $query = substr($query, 0, $p) . "CONVERT(char(19), " . substr($query, $p, strlen($daterow)) . ", 20) AS $daterow" . substr($query, $p + strlen($daterow));
        else
          $query = substr($query, 0, $p) . "CONVERT(char(19), " . substr($query, $p, strlen($daterow)) . ", 20)" . substr($query, $p + strlen($daterow));
      }
    }
  }
*/

  // Replace USE INDEX
  if (($ui = strpos($query, "USE INDEX")) !== false) {
    if (($ep = strpos(substr($query, $ui + 9), ")")) !== false) {
      $tmp = substr($query, $ui, $ep + 10); // USE INDEX (ge_kstype)
      $tmp = preg_replace("/USE INDEX/", "WITH(INDEX", $tmp) . ")";
      $query = substr($query, 0, $ui) . $tmp . substr($query, $ui + $ep + 10);
    }
  }

  return $query;
}

?>